@@ -6,10 +6,12 @@ use Pcore::AE::Handle qw[:PERSISTENT :PROXY_TYPE];
 use Pcore::Util::Scalar qw[refaddr];
 use Compress::Raw::Zlib qw[WANT_GZIP_OR_ZLIB Z_OK Z_STREAM_END];
 
-const our $CONTENT_ENCODING_GZIP     => 1;    # NOTE only gzip is supported now
-const our $CONTENT_ENCODING_COMPRESS => 2;
-const our $CONTENT_ENCODING_DEFLATE  => 3;
-const our $CONTENT_ENCODING_IDENTITY => 4;
+# https://en.wikipedia.org/wiki/HTTP_compression
+const our $CONTENT_ENCODING_IDENTITY => 0;    # no compression
+const our $CONTENT_ENCODING_GZIP     => 1;    # implemented
+const our $CONTENT_ENCODING_DEFLATE  => 2;    # not supported
+const our $CONTENT_ENCODING_COMPRESS => 3;    # deprecated, not supported
+const our $CONTENT_ENCODING_BROTLI   => 4;    # new method, need to be implemented
 
 # $status => $switch_method_to_GET
 const our $REDIRECT => {
@@ -66,7 +68,8 @@ sub http_request ($args) {
                     }
                 }
 
-                $persistent ? $runtime->{h}->store : $runtime->{h}->destroy;
+                # store or destroy handle
+                $persistent ? $runtime->{h}->store( $args->{keepalive_timeout} ) : $runtime->{h}->destroy;
 
                 # process redirect
                 if ( $runtime->{redirect} ) {
@@ -149,8 +152,8 @@ sub http_request ($args) {
 
             # prepare handle
             $h->on_error(
-                sub ( $h, $fatal, $message ) {
-                    $runtime->{finish}->( $runtime->{on_error_status}, $message );
+                sub ( $h, $fatal, $reason ) {
+                    $runtime->{finish}->( $runtime->{on_error_status}, $reason );
 
                     return;
                 }
@@ -206,18 +209,18 @@ sub _connect ( $args, $runtime, $cb ) {
         tls_ctx                => $args->{tls_ctx},
         bind_ip                => $args->{bind_ip},
         proxy                  => $args->{proxy},
-        on_proxy_connect_error => sub ( $h, $message, $proxy_error ) {
-            $runtime->{finish}->( 594, $message );
+        on_proxy_connect_error => sub ( $h, $reason, $proxy_error ) {
+            $runtime->{finish}->( 594, $reason );
 
             return;
         },
-        on_connect_error => sub ( $h, $message ) {
-            $runtime->{finish}->( $runtime->{on_error_status}, $message );
+        on_connect_error => sub ( $h, $reason ) {
+            $runtime->{finish}->( $runtime->{on_error_status}, $reason );
 
             return;
         },
-        on_error => sub ( $h, $fatal, $message ) {
-            $runtime->{finish}->( $runtime->{on_error_status}, $message );
+        on_error => sub ( $h, $fatal, $reason ) {
+            $runtime->{finish}->( $runtime->{on_error_status}, $reason );
 
             return;
         },
@@ -268,11 +271,17 @@ sub _write_request ( $args, $runtime ) {
         delete $args->{headers}->{TRANSFER_ENCODING};
     }
 
+    # serialize headers
     my $headers = $args->{headers}->to_string;
 
-    # prepare basic authorization
-    if ( my $userinfo_b64 = $args->{url}->userinfo_b64 ) {
-        $headers .= 'Authorization: Basic ' . $userinfo_b64 . $CRLF;
+    # add basic authorization
+    if ( !$args->{headers}->{AUTHORIZATION} && ( my $userinfo_b64 = $args->{url}->userinfo_b64 ) ) {
+        $headers .= 'Authorization:Basic ' . $userinfo_b64 . $CRLF;
+    }
+
+    # close connection, if not persistent
+    if ( !$args->{persistent} && !$args->{headers}->{CONNECTION} ) {
+        $headers .= 'Connection:close' . $CRLF;
     }
 
     # send request headers
@@ -337,7 +346,7 @@ sub _read_headers ( $args, $runtime, $cb ) {
                         $runtime->{redirect} = 1;
 
                         # create new response object and set it as default response for current request
-                        $runtime->{res} = Pcore::HTTP::Response->new;
+                        $runtime->{res} = Pcore::HTTP::Response->new( { status => $res->{status} } );
 
                         push $args->{res}->redirect->@*, $runtime->{res};
                     }
@@ -353,7 +362,7 @@ sub _read_headers ( $args, $runtime, $cb ) {
                 # fill response object with HTTP response headers data
                 $runtime->{res}->{headers} = $res->{headers};
 
-                $runtime->{res}->set_version( q[1.] . $res->{minor_version} );
+                $runtime->{res}->set_version("1.$res->{minor_version}");
 
                 $runtime->{res}->set_status( $res->{status}, $res->{reason} );
             }
@@ -561,7 +570,8 @@ sub _read_body ( $args, $runtime, $cb ) {
                     if ( !$decode->( $content_ref, $out_buf_ref ) ) {
                         $cb->( 597, 'Stream decode error' );
 
-                        return;    # stop reading
+                        # stop reading
+                        return;
                     }
                 }
                 elsif ( defined $content_ref ) {
@@ -599,9 +609,12 @@ sub _read_body ( $args, $runtime, $cb ) {
 
                     $args->{on_progress}->( $runtime->{res}, $runtime->{content_length}, $total_bytes_readed ) if $args->{on_progress};
 
-                    return 1;    # continue reading
+                    # continue reading
+                    return 1;
                 }
-                else {           # last chunk
+
+                # last chunk
+                else {
                     $cb->();
                 }
             }
@@ -610,13 +623,18 @@ sub _read_body ( $args, $runtime, $cb ) {
         };
     }
 
-    if ($chunked) {              # read chunked body
+    # read chunked body
+    if ($chunked) {
         $runtime->{h}->read_http_body( $on_read, chunked => 1, headers => $runtime->{res}->headers );
     }
-    elsif ( $runtime->{content_length} ) {    # read body with known content length
+
+    # read body with known content length
+    elsif ( $runtime->{content_length} ) {
         $runtime->{h}->read_http_body( $on_read, length => $runtime->{content_length} );
     }
-    else {                                    # read body with unknown content length (until EOF)
+
+    # read body with unknown content length (until EOF)
+    else {
         $runtime->{h}->read_http_body($on_read);
     }
 
@@ -631,15 +649,13 @@ sub _read_body ( $args, $runtime, $cb ) {
 ## | Sev. | Lines                | Policy                                                                                                         |
 ## |======+======================+================================================================================================================|
 ## |    3 |                      | Subroutines::ProhibitExcessComplexity                                                                          |
-## |      | 23                   | * Subroutine "http_request" with high complexity score (28)                                                    |
-## |      | 241                  | * Subroutine "_write_request" with high complexity score (22)                                                  |
-## |      | 370                  | * Subroutine "_read_body" with high complexity score (67)                                                      |
+## |      | 25                   | * Subroutine "http_request" with high complexity score (28)                                                    |
+## |      | 244                  | * Subroutine "_write_request" with high complexity score (25)                                                  |
+## |      | 379                  | * Subroutine "_read_body" with high complexity score (67)                                                      |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    3 | 103, 117, 119, 200   | References::ProhibitDoubleSigils - Double-sigil dereference                                                    |
+## |    3 | 359                  | ErrorHandling::RequireCheckingReturnValueOfEval - Return value of eval not tested                              |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    3 | 350                  | ErrorHandling::RequireCheckingReturnValueOfEval - Return value of eval not tested                              |
-## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    3 | 581                  | ControlStructures::ProhibitDeepNests - Code structure is deeply nested                                         |
+## |    3 | 591                  | ControlStructures::ProhibitDeepNests - Code structure is deeply nested                                         |
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ##
 ## -----SOURCE FILTER LOG END-----
